@@ -163,31 +163,25 @@ async fn next_entry(handle: Arc<Handle>, operation: Operation, destination: Opti
 
 	let read_result: Result<(Code, HeaderData), Error> = {
 		let handle = handle.clone();
-		let header = HeaderData::default();
+		let mut header = HeaderData::default();
 		spawn_blocking(move || unsafe {
-			let mut header: unrar_sys::HeaderData = header.try_into()?;
-			let code = Code::try_from(unrar_sys::RARReadHeader(handle.as_ffi(), &mut header as *mut _) as u32)?;
-			Ok((code, header.try_into()?))
+			let code = Code::try_from(unrar_sys::RARReadHeader(handle.as_ffi(), &mut header as *mut _ as *mut _) as u32)?;
+			Ok((code, header))
 		}).await?
 	};
-	let (code, mut header) = read_result?;
+	let (code, header) = read_result?;
 
 	let process_result = match code {
 		Code::Success => {
-			// TODO:  Why does reading archive_name.as_ptr() solve the late return with incorrect code?  UB?
-			// TODO:  It gets better; the raw pointer makes the whole Future !Send, so we have to wrap it in fragile!
-			let _p = fragile::Fragile::new(header.archive_name.as_ptr());
-			// TODO:  Why does moving header into the blocking thread and back out solve that segfault problem?  Quirk of RARProcessFile() or are we causing UB somehow?
-			let (result, hdr) = spawn_blocking(move || unsafe {
+			let result = spawn_blocking(move || unsafe {
 				let code = unrar_sys::RARProcessFile(
 					handle.as_ffi(),
 					operation as i32,
 					destination.as_ref().map(|s| s.as_ptr() as *const _).unwrap_or(ptr::null()),
 					ptr::null()
 				) as u32;
-				(Code::try_from(code), header)
+				Code::try_from(code)
 			}).await?;
-			header = hdr;
 			Ok(result?)
 		},
 		Code::EndArchive => Err(RarError::EndArchive),
